import Vue from 'vue';

function _classCallCheck(instance, Constructor) {
  if (!(instance instanceof Constructor)) {
    throw new TypeError("Cannot call a class as a function");
  }
}

function _defineProperties(target, props) {
  for (var i = 0; i < props.length; i++) {
    var descriptor = props[i];
    descriptor.enumerable = descriptor.enumerable || false;
    descriptor.configurable = true;
    if ("value" in descriptor) descriptor.writable = true;
    Object.defineProperty(target, descriptor.key, descriptor);
  }
}

function _createClass(Constructor, protoProps, staticProps) {
  if (protoProps) _defineProperties(Constructor.prototype, protoProps);
  if (staticProps) _defineProperties(Constructor, staticProps);
  return Constructor;
}

function _toConsumableArray(arr) {
  return _arrayWithoutHoles(arr) || _iterableToArray(arr) || _unsupportedIterableToArray(arr) || _nonIterableSpread();
}

function _arrayWithoutHoles(arr) {
  if (Array.isArray(arr)) return _arrayLikeToArray(arr);
}

function _iterableToArray(iter) {
  if (typeof Symbol !== "undefined" && Symbol.iterator in Object(iter)) return Array.from(iter);
}

function _unsupportedIterableToArray(o, minLen) {
  if (!o) return;
  if (typeof o === "string") return _arrayLikeToArray(o, minLen);
  var n = Object.prototype.toString.call(o).slice(8, -1);
  if (n === "Object" && o.constructor) n = o.constructor.name;
  if (n === "Map" || n === "Set") return Array.from(n);
  if (n === "Arguments" || /^(?:Ui|I)nt(?:8|16|32)(?:Clamped)?Array$/.test(n)) return _arrayLikeToArray(o, minLen);
}

function _arrayLikeToArray(arr, len) {
  if (len == null || len > arr.length) len = arr.length;

  for (var i = 0, arr2 = new Array(len); i < len; i++) arr2[i] = arr[i];

  return arr2;
}

function _nonIterableSpread() {
  throw new TypeError("Invalid attempt to spread non-iterable instance.\nIn order to be iterable, non-array objects must have a [Symbol.iterator]() method.");
}

function isHtmlElement(el) {
  return el instanceof HTMLElement;
}
function isVueComponent(el) {
  var elem = el;
  return elem.__vue__ !== undefined;
} // helper method to add and remove classes from an element

function addAndRemoveCssClass(el, addList, removeList) {
  if (addList !== undefined && addList.length !== 0) {
    var _el$classList;

    (_el$classList = el.classList).add.apply(_el$classList, _toConsumableArray(addList));
  }

  if (removeList !== undefined && removeList.length !== 0) {
    var _el$classList2;

    (_el$classList2 = el.classList).remove.apply(_el$classList2, _toConsumableArray(removeList));
  }
}

function prepareAndWatch(el, binding, node) {
  var element = el;
  element.isFromDirective = true;
  element.binding = binding;

  if (isHtmlElement(el)) {
    node.context.$lazyObserver.startObserving(element);
  } else if (isVueComponent(el)) {
    element.__vue__.$lazyObserver.startObserving(element);
  }
}

var lazyAnimateDirective = function lazyAnimateDirective(app) {
  return app.directive('lazytransition', {
    bind: function bind(el, binding, vnode) {
      prepareAndWatch(el, binding, vnode);
    }
  });
};
var lazyAnimateGroup = function lazyAnimateGroup(app) {
  return app.directive('lazytransition-group', {
    bind: function bind(el, binding, vnode) {
      if (el.hasChildNodes()) {
        var numberOfChildren = el.children.length;

        for (var i = 0; i < numberOfChildren; i++) {
          var elm = el.children.item(i);
          prepareAndWatch(elm, binding, vnode);
        }
      }
    }
  });
};

var script = Vue.extend({
  name: 'LazyTransition',
  // vue component name
  props: {
    name: {
      required: false,
      default: ''
    },
    height: {
      required: false,
      default: 200
    }
  },
  data: function data() {
    return {
      show: false
    };
  },
  mounted: function mounted() {
    var el = this.$refs.animateditem;
    this.$lazyObserver.startObserving(el);
  },
  beforeDestroy: function beforeDestroy() {
    var el = this.$refs.animateditem;
    this.$lazyObserver.stopObserving(el);
  }
});

function normalizeComponent(template, style, script, scopeId, isFunctionalTemplate, moduleIdentifier /* server only */, shadowMode, createInjector, createInjectorSSR, createInjectorShadow) {
    if (typeof shadowMode !== 'boolean') {
        createInjectorSSR = createInjector;
        createInjector = shadowMode;
        shadowMode = false;
    }
    // Vue.extend constructor export interop.
    const options = typeof script === 'function' ? script.options : script;
    // render functions
    if (template && template.render) {
        options.render = template.render;
        options.staticRenderFns = template.staticRenderFns;
        options._compiled = true;
        // functional template
        if (isFunctionalTemplate) {
            options.functional = true;
        }
    }
    // scopedId
    if (scopeId) {
        options._scopeId = scopeId;
    }
    let hook;
    if (moduleIdentifier) {
        // server build
        hook = function (context) {
            // 2.3 injection
            context =
                context || // cached call
                    (this.$vnode && this.$vnode.ssrContext) || // stateful
                    (this.parent && this.parent.$vnode && this.parent.$vnode.ssrContext); // functional
            // 2.2 with runInNewContext: true
            if (!context && typeof __VUE_SSR_CONTEXT__ !== 'undefined') {
                context = __VUE_SSR_CONTEXT__;
            }
            // inject component styles
            if (style) {
                style.call(this, createInjectorSSR(context));
            }
            // register component module identifier for async chunk inference
            if (context && context._registeredComponents) {
                context._registeredComponents.add(moduleIdentifier);
            }
        };
        // used by ssr in case component is cached and beforeCreate
        // never gets called
        options._ssrRegister = hook;
    }
    else if (style) {
        hook = shadowMode
            ? function (context) {
                style.call(this, createInjectorShadow(context, this.$root.$options.shadowRoot));
            }
            : function (context) {
                style.call(this, createInjector(context));
            };
    }
    if (hook) {
        if (options.functional) {
            // register for functional component in vue file
            const originalRender = options.render;
            options.render = function renderWithStyleInjection(h, context) {
                hook.call(context);
                return originalRender(h, context);
            };
        }
        else {
            // inject component registration as beforeCreate hook
            const existing = options.beforeCreate;
            options.beforeCreate = existing ? [].concat(existing, hook) : [hook];
        }
    }
    return script;
}

/* script */
var __vue_script__ = script;
/* template */

var __vue_render__ = function __vue_render__() {
  var _vm = this;

  var _h = _vm.$createElement;

  var _c = _vm._self._c || _h;

  return _c('div', {
    ref: "animateditem",
    staticClass: "lazy-animated",
    style: {
      minHeight: _vm.height + 'px'
    }
  }, [_c('transition', {
    attrs: {
      "name": _vm.name
    }
  }, [_vm.show ? _vm._t("default", [_vm._v(" DEFAULT ")]) : _vm._e()], 2)], 1);
};

var __vue_staticRenderFns__ = [];
/* style */

var __vue_inject_styles__ = undefined;
/* scoped */

var __vue_scope_id__ = undefined;
/* module identifier */

var __vue_module_identifier__ = undefined;
/* functional template */

var __vue_is_functional_template__ = false;
/* style inject */

/* style inject SSR */

/* style inject shadow dom */

var __vue_component__ = normalizeComponent({
  render: __vue_render__,
  staticRenderFns: __vue_staticRenderFns__
}, __vue_inject_styles__, __vue_script__, __vue_scope_id__, __vue_is_functional_template__, __vue_module_identifier__, false, undefined, undefined, undefined);

// add and remove vue transition classes
function addTransitionEvents(el, binding, state) {
  var element = el;
  var className = binding.value;
  var transition = className + '-' + state;
  var transitionActive = transition + '-active';
  var transitionTo = transition + '-to';
  addAndRemoveCssClass(element, [transition]);
  element.addEventListener('transitionstart', function () {
    addAndRemoveCssClass(element, [transitionTo], [transition]);
  }, {
    once: true
  });
  element.addEventListener('transitionrun', function () {
    addAndRemoveCssClass(element, [transitionActive]);
  }, {
    once: true
  });
  element.addEventListener('transitionend', function () {
    addAndRemoveCssClass(element, undefined, [transitionActive, transition, transitionTo]);
  }, {
    once: true
  });
} // Run when element is in view


function handleElementInView(entry, intersectionRatio, config, observerInstance) {
  // get the element
  var vueElement = entry.target;
  config.root; // to be used later for more customisation

  if (entry.isIntersecting) {
    // if element is intersecting
    if (entry.intersectionRatio >= intersectionRatio) {
      // and more than the specified percentage is in view
      if (isVueComponent(vueElement)) {
        // handle component syntax
        vueElement.__vue__.$data.show = true;
        vueElement.removeAttribute('style');
      }

      if (vueElement.isFromDirective) {
        addAndRemoveCssClass(vueElement, [vueElement.binding.value + '-enter-active'], [vueElement.binding.value + '-enter']);
      } // Stop observing after object is in view and transition is done


      observerInstance.stopObserving(vueElement);
    }
  }
}

var ObserverService = /*#__PURE__*/function () {
  function ObserverService(config) {
    var _this = this;

    _classCallCheck(this, ObserverService);

    this._observer = new IntersectionObserver(function (entries) {
      entries.forEach(function (entry) {
        handleElementInView(entry, config.intersectionRatio, config.options, _this);
      });
    }, config.options);
  }

  _createClass(ObserverService, [{
    key: "startObserving",
    value: function startObserving(el) {
      if (el.isFromDirective) addTransitionEvents(el, el.binding, 'enter');
      this.observer.observe(el);
    }
  }, {
    key: "stopObserving",
    value: function stopObserving(el) {
      this.observer.unobserve(el);
    }
  }, {
    key: "disposeObserver",
    value: function disposeObserver() {
      this.observer.disconnect();
    }
  }, {
    key: "observer",
    get: function get() {
      return this._observer;
    }
  }]);

  return ObserverService;
}();

var defaultObserverConfig = {
  options: {
    root: null,
    rootMargin: '0px',
    threshold: [0.5, 0.75, 1]
  },
  intersectionRatio: 0
};

function initConfig(config) {
  config.options.root = config.options.root === undefined ? config.options.root = defaultObserverConfig.options.root : config.options.root;
  config.options.rootMargin = config.options.rootMargin === undefined ? config.options.rootMargin = defaultObserverConfig.options.rootMargin : config.options.rootMargin;
  config.options.threshold = config.options.threshold === undefined ? config.options.threshold = defaultObserverConfig.options.threshold : config.options.threshold;
  return config;
}

function createObserverService(config) {
  if (config !== undefined) {
    config = initConfig(config);
    return new ObserverService(config);
  } else {
    return new ObserverService(defaultObserverConfig);
  }
}

// install function executed by Vue.use()
var install = function installLazyAnimate(Vue, config) {
  if (install.installed) return; // add global variable

  Vue.prototype.$lazyObserver = createObserverService(config);
  install.installed = true; // Add Component

  Vue.component('LazyTransition', __vue_component__); // Add Directive

  lazyAnimateDirective(Vue);
  lazyAnimateGroup(Vue);
}; // Create module definition for Vue.use()


var LazyTransitionPlugin = {
  install: install
}; // To auto-install when vue is found
// eslint-disable-next-line no-redeclare

/* global window, global */

var GlobalVue = null;

if (typeof window !== 'undefined') {
  GlobalVue = window.Vue;
} else if (typeof global !== 'undefined') {
  // eslint-disable-next-line @typescript-eslint/no-explicit-any
  GlobalVue = global.Vue;
}

if (GlobalVue) {
  GlobalVue.use(LazyTransitionPlugin);
} // Inject install function into component - allows component
// to be registered via Vue.use() as well as Vue.component()
// eslint-disable-next-line @typescript-eslint/no-explicit-any


__vue_component__.install = install; //
// also be used as directives, etc. - eg. import { RollupDemoDirective } from 'rollup-demo';
// export const RollupDemoDirective = component;

export default __vue_component__;
